@@ -5,7 +5,7 @@
 /*
  * function prototype
  */
-void rgb_bitdist(Test **test,int irun);
+int rgb_bitdist(Test **test,int irun);
 
 static Dtest rgb_bitdist_dtest = {
   "RGB Bit Distribution Test",
@@ -20,9 +20,18 @@ static Dtest rgb_bitdist_dtest = {
 # WITHOUT overlap (e.g. 01|10|10|01|11|00|01|10) so the samples\n\
 # are independent.  Every other sample is offset modulus of the\n\
 # sample index and ntuple_max.\n\
+#\n\
+# This test must be run with -n ntuple for ntuple > 0.  Note that if\n\
+# ntuple > 12, one should probably increase tsamples so that each of the\n\
+# 2^ntuple bins should end up with an average of around 30 occurrences.\n\
+# Note also that the memory requirements and CPU time requirements will\n\
+# get quite large by e.g. ntuple = 20 -- use caution when sampling the\n\
+# distribution of very large ntuples.\n\
 #\n",
   100,     /* Default psamples */
   100000,  /* Default tsamples */
-  1        /* We magically make all the bit tests return a single histogram */
+  1,       /* We magically make all the bit tests return a single histogram */
+  rgb_bitdist,
+  0
 };
 
