@@ -13,13 +13,13 @@
 
 #define RGB_PERM_KMAX 10
 uint nperms;
-uint rgb_perm(size_t *data,int len);
 double fpipi(int pi1,int pi2,int nkp);
+uint rgb_permutations_k;
 
-void rgb_permutations(Test **test,int irun)
+int rgb_permutations(Test **test,int irun)
 {
 
- uint i,j,permindex,t;
+ uint i,j,k,permindex=0,t;
  Vtest vtest;
  double *testv;
  size_t ps[4096];
@@ -32,23 +32,31 @@ void rgb_permutations(Test **test,int irun)
  }
 
  /*
-  * Number of permutations
+  * Number of permutations.  Note that the minimum ntuple value for a
+  * valid test is 2.  If ntuple is less than 2, we choose the default
+  * test size as 5 (like operm5).
   */
- nperms = gsl_sf_fact(rgb_permutations_k);
+ if(ntuple<2){
+   test[0]->ntuple = 5;
+ } else {
+   test[0]->ntuple = ntuple;
+ }
+ k = test[0]->ntuple;
+ nperms = gsl_sf_fact(k);
 
  /*
   * A vector to accumulate rands in some sort order
   */
- testv = (double *)malloc(rgb_permutations_k*sizeof(double));
+ testv = (double *)malloc(k*sizeof(double));
 
  MYDEBUG(D_RGB_PERMUTATIONS){
-   printf("# rgb_permutations: There are %u permutations of length k = %u\n",nperms,rgb_permutations_k);
+   printf("# rgb_permutations: There are %u permutations of length k = %u\n",nperms,k);
  }
 
  /*
   * Create a test, initialize it.
   */
- Vtest_create(&vtest,nperms,"rgb_permutations",gsl_rng_name(rng));
+ Vtest_create(&vtest,nperms);
  vtest.cutoff = 5.0;
  for(i=0;i<nperms;i++){
    vtest.x[i] = 0.0;
@@ -60,7 +68,7 @@ void rgb_permutations(Test **test,int irun)
  }
  lookup = (gsl_permutation**) malloc(nperms*sizeof(gsl_permutation*));
  for(i=0;i<nperms;i++){
-   lookup[i] = gsl_permutation_alloc(rgb_permutations_k);
+   lookup[i] = gsl_permutation_alloc(k);
  }
  for(i=0;i<nperms;i++){
    if(i == 0){
@@ -94,23 +102,23 @@ void rgb_permutations(Test **test,int irun)
    /*
     * To sort into a perm, test vector needs to be double.
     */
-   for(i=0;i<rgb_permutations_k;i++) {
+   for(i=0;i<k;i++) {
      testv[i] = (double) gsl_rng_get(rng);
      MYDEBUG(D_RGB_PERMUTATIONS){
        printf("# rgb_permutations: testv[%u] = %u\n",i,(uint) testv[i]);
      }
    }
 
-   gsl_sort_index(ps,testv,1,rgb_permutations_k);
+   gsl_sort_index(ps,testv,1,k);
 
    MYDEBUG(D_RGB_PERMUTATIONS){
-     for(i=0;i<rgb_permutations_k;i++) {
-       printf("# rgb_permutations: ps[%u] = %u\n",i,ps[i]);
+     for(i=0;i<k;i++) {
+       printf("# rgb_permutations: ps[%u] = %lu\n",i,ps[i]);
      }
    }
 
    for(i=0;i<nperms;i++){
-     if(memcmp(ps,lookup[i]->data,rgb_permutations_k*sizeof(size_t))==0){
+     if(memcmp(ps,lookup[i]->data,k*sizeof(size_t))==0){
        permindex = i;
        MYDEBUG(D_RGB_PERMUTATIONS){
          printf("# Found permutation: ");
@@ -143,33 +151,13 @@ void rgb_permutations(Test **test,int irun)
  }
 
  for(i=0;i<nperms;i++){
-   free(lookup[i]);
+   gsl_permutation_free(lookup[i]);
  }
  free(lookup);
  free(testv);
+ Vtest_destroy(&vtest);
 
-}
-
-uint rgb_perm(size_t *data,int len)
-{
-
- uint i,j,k,max,min;
- uint pindex,uret,tmp;
- static gsl_permutation** lookup = 0;
-
- MYDEBUG(D_RGB_PERMUTATIONS){
-   printf("# rgb_permutations: Entering rgb_perm.  Going to match up:\n");  
-   for(i=0;i<len;i++){
-     printf("# rgb_permutations: data[%u] = %u\n",i,data[i]);
-   }
- }
- /*
-  * Allocate space for lookup table and fill it.
-  */
- if(lookup == 0){
- }
-
- printf("We'd better not get here...\n");
+ return(0);
 
 }
 
